#!/usr/bin/python3

import os
import json
import logging
import collections
from gi.repository import Gio
from gettext import gettext as _
from SystemUpdater.Core.errors import *
from SystemUpdater.Core.enums import *
from gi.repository import GObject, GLib

class Transaction():
    # 更新的缓存数据保存位置 
    UPDATE_AVAILABLE_DATA = "/var/cache/kylin-system-updater/update_available_data.json"

    def __init__(self,tid=None, role=None, sender=None):
        self.from_revision = None
        self.new_revision = None
        self.available_refs = None
        self.deployed_refs = None
        self.available_repo = None
        self.push_list = []
        self.available_metadata = {}
        self.tid = tid
        self.role = ROLE_UNSET
        self.sender = sender
        self._deployable = False

        if os.path.exists(self.UPDATE_AVAILABLE_DATA):
            self.read_local()

        def cancelled():
            logging.info("Task has been cancelled...")
        self.cancellable = Gio.Cancellable.new()
        self.cancellable.connect(cancelled)

    def _write_local(self):
        try:
            output_upgrade = {}
            output_upgrade.update({"from_revision":self.from_revision})
            output_upgrade.update({"new_revision":self.new_revision})
            output_upgrade.update({"available_refs":self.available_refs})
            output_upgrade.update({"deployed_refs":self.deployed_refs})
            output_upgrade.update({"available_repo":self.available_repo})
            # output_upgrade.update({"push_list":self.push_list})
            output_upgrade.update({"available_metadata":self.available_metadata})
            output_upgrade.update({"_deployable":self._deployable})

            #6 产生JSON文件
            with open(self.UPDATE_AVAILABLE_DATA, 'w', encoding='utf-8') as f:
                json.dump(output_upgrade, f, ensure_ascii=False, indent=4)
            logging.info("Write Upgrade data to local json(%s) in shutdown model...",self.UPDATE_AVAILABLE_DATA)
        except Exception as exc:
            logging.error(exc)
    def read_local(self):
        #读取组JSON文件
        try:
            with open(self.UPDATE_AVAILABLE_DATA,'r') as f:
                data = json.load(f)
                self.from_revision = data.get("from_revision",None)
                self.new_revision = data.get("new_revision",None)
                self.available_refs = data.get("available_refs",None)
                self.deployed_refs = data.get("deployed_refs",None)
                self.available_repo = data.get("available_repo",None)
                # self.push_list = data.get("push_list",[])
                self.available_metadata = data.get("available_metadata",{})
                self._deployable = data.get("_deployable",False)
            logging.info("Finished: reading upgrade data from the local file...")
        except Exception as exc:
            logging.error(exc)

    def _get_deployable(self):
        return self._deployable

    def _set_deployable(self, state):
        self._deployable = state
        self._write_local()

    def _set_default(self,state):
        pass

    def _get_download_size(self):
        return self.available_metadata.get("download-size",0)

    def _get_install_size(self):
        return self.available_metadata.get("install-size",0)

    deployable = property(_get_deployable, _set_deployable)
    dowanload_size = property(_get_download_size, _set_default)
    install_size = property(_get_install_size, _set_default)

class TransactionQueue(GObject.GObject):

    """Queue for transactions."""

    __gsignals__ = {"queue-changed": (GObject.SignalFlags.RUN_FIRST,
                                      None,
                                      ())}

    def __init__(self, worker):
        """Intialize a new TransactionQueue instance."""
        GObject.GObject.__init__(self)
        self._queue = collections.deque()
        self._proc_count = 0
        self.worker = worker
        # Used to keep track of not yet queued transactions
        self.limbo = {}
        # self.worker.connect("transaction-done", self._on_transaction_done)

    def __len__(self):
        return len(self._queue)

    def _emit_queue_changed(self):
        """Emit the queued-changed signal."""
        logging.debug("emitting queue changed")
        self.emit("queue-changed")

    def put(self, tid):
        """Add an item to the queue."""
        trans = self.limbo.pop(tid)
        if trans._idle_watch is not None:
            GLib.source_remove(trans._idle_watch)
        if self.worker.trans:
            trans.status = enums.STATUS_WAITING
            self._queue.append(trans)
        else:
            self.worker.run(trans)
        self._emit_queue_changed()

    # def _on_transaction_done(self, worker, trans):
    #     """Mark the last item as done and request a new item."""
    #     # FIXME: Check if the transaction failed because of a broken system or
    #     #       if dpkg journal is dirty. If so allready queued transactions
    #     #       except the repair transactions should be removed from the queue
    #     if trans.exit in [enums.EXIT_FAILED, enums.EXIT_CANCELLED]:
    #         if trans.exit == enums.EXIT_FAILED:
    #             exit = enums.EXIT_PREVIOUS_FAILED
    #         else:
    #             exit = enums.EXIT_CANCELLED
    #         _trans = trans.after
    #         while _trans:
    #             self.remove(_trans)
    #             _trans.exit = exit
    #             msg = enums.get_role_error_from_enum(trans.role)
    #             _trans.status_details = msg
    #             _trans = _trans.after
    #     try:
    #         next_trans = self._queue.popleft()
    #     except IndexError:
    #         logging.debug("There isn't any queued transaction")
    #     else:
    #         self.worker.run(next_trans)
    #     self._emit_queue_changed()

    def remove(self, transaction):
        """Remove the specified item from the queue."""
        self._queue.remove(transaction)
        self._emit_queue_changed()

    def clear(self):
        """Remove all items from the queue."""
        for transaction in self._queue:
            transaction._remove_from_connection_no_raise()
        self._queue.clear()

    @property
    def items(self):
        """Return a list containing all queued items."""
        return list(self._queue)